% Function for the Integrated  CoSpectrum;
% This code computes the Integrated  CoSpectrum as described in
%%----------------------------------------------
%%Pancrazi 2014 ("The Heterogenous Great Moderation", the European Economic
%Review)
%%-----------------------------------------------------------------
% ****** Please refer to that paper in case you are using these codes.******
%%-----------------------------------------------------------------

%
%   INPUT: Y MATRIX OF SERIES (in column)
%          wL,wU= interval of the frequency of interest 
%
% Example: Business Cycle frequency:   wl=pi/16, wh= pi/13


function[IntCo]=IntegrCoSpec(Y,wL,wU);
[T,k]=size(Y);



% autoCov Matrix
mu=zeros(k,k);
muY=ones(T,k);
for i=1:k
    mu(i,i)=mean(Y(:,i));
end;
muY=muY*mu;

%muY=zeros(T,k);


G0=(1/T)* (Y-muY)'*(Y-muY);
SUM=zeros(k,k);
SUMm=zeros(k,k);
for j=1:T-1    
    Y_j=lagmatrix(Y,j);    
    Ytemp=Y(j+1:end,:);
    Y_jtemp=Y_j(j+1:end,:);
    muYtemp=muY(j+1:end,:);
    Gj=(1/T)*(Ytemp-muYtemp)'*(Y_jtemp-muYtemp);
    SUM=SUM+ Gj*(sin(wU*j)-sin(wL*j))/j;
    
    Gjm=(1/T)*(Y_jtemp-muYtemp)'*(Ytemp-muYtemp);
    SUMm=SUMm+ Gjm*(sin(-wL*j)-sin(-wU*j))/j;
end;
IntCo=(G0*(wU-wL)/pi)+(SUM+SUMm)/pi; 

1;

