%This code computes the Spectral Covariance Instability Tests proposed in:
%%----------------------------------------------
%%Pancrazi 2014 ("The Heterogenous Great Moderation", the European Economic
%Review)
%%-----------------------------------------------------------------
% ****** Please refer to that paper in case you are using these codes.******
%%-----------------------------------------------------------------



% INPUT: Y= vector of series of interests. The series have to be of the
% same lengths.
% wL = lower frequency of interest
% wU = higher frequencey of interest
% N  = natural number that indicates in how many discrete intervals the
% integral is approximated. Example: N=200. The higher N, the higher is the
% computational time and the
% more accurate is the solution is
% stepTao= step value for each tao. Example: 0.01. 

%OUTPUT:
%W_APL      =  Spectral Average Wald Test statistics (for the kxk matrix: the element i,j represent
%              the  Spectral Average Wald Test statistics for the covariance betwwen Y(i) and Y(j).
%W_EXP      = Spectral Exponential Wald Test  (for the kxk matrix: the element i,j represent
%              the Spectral Exponential Wald Test statistics for the covariance betwwen Y(i) and Y(j).
%W_Nyb       = Spectral Nyblom Test statistics (for the kxk matrix: the element i,j represent
%              the Spectral Nyblom Test for the covariance betwwen Y(i) and Y(j).

%pvaluesAPL     = pvalues for the Spectral Average Wald Test 
%pvaluesEXP     = pvalues for the Spectral Exponential Wald Test
%pvaluesNYB     = pvalues for the Spectral Nyblom Wald Test

%IMPORTANT: Given the symmetry, the test computes only the upper-triangolar
%part of the relevant matrices. The lower-triangolar part is filled with
%NaN for speed improvement


function [W_APL,W_EXP,W_NYB,pvaluesAPL,pvaluesEXP,pvaluesNYB]=Sci_Tests_Function(Y,wL,wU,N,stepTao)

[T,k]=size(Y);
count=0;
taohowmany=length((0.15:stepTao:0.85)');
taocount=zeros(taohowmany,1);
tbcount=taocount;
W=zeros(k,k,taohowmany);
WNy=zeros(k,k,taohowmany);
W_APL=zeros(k,k);
W_EXP=zeros(k,k);
W_NYB=zeros(k,k);


for tao=0.15:stepTao:0.85  
    tao;
    count=count+1;
    tb=round(T*tao);
    taocount(count,1)=tao;
    tbcount(count,1)=tb;

    y1=Y(1:tb,:);
    y2=Y(tb+1:end,:);
    
    [I1]=IntegrCoSpec(y1,wL,wU);
    [V1]=VarIntCoSpec(y1,wL,wU,N);
    
    [I2]=IntegrCoSpec(y2,wL,wU);
    [V2]=VarIntCoSpec(y2,wL,wU,N);
    
    for i=1:k
        for j=i:k
            W(i,j,count)=T*(I1(i,j)-I2(i,j))'/( V1(i,j)/tao + V2(i,j)/(1-tao)) *(I1(i,j)-I2(i,j));
            WNy(i,j,count)=W(i,j,count)*tao*(1-tao);
        end
    end

end;



TW=taohowmany;
    
for i=1:k   
    for j=1:k
        if j<i
             W_APL(i,j)=NaN;
             W_EXP(i,j)=NaN;
             W_NYB(i,j)=NaN;
        else           
        W_APL(i,j)=mean(W(i,j,:));
        W_EXP(i,j)=log(sum(exp(1/2*W(i,j,:))*(1/TW)));
        W_NYB(i,j)=mean(WNy(i,j,:));
        end
    end
end


%FIND THE P_VALUES OF THE TEST

load pvapisb.txt; load pvap0sb.txt; load pvqlrsb.txt; load pvnybsb.txt;

 for i=1:k   
    for j=1:k
         if j<i
             pvaluesAPL(i,j)=NaN;
             pvaluesEXP(i,j)=NaN;
             pvaluesNYB(i,j)=NaN;
         else 
            pvaluesAPL(i,j)=pvcalc(W_APL(i,j),pvap0sb,0);
            pvaluesEXP(i,j)=pvcalc(W_EXP(i,j),pvapisb,0);
            pvaluesNYB(i,j)=pvcalc(W_NYB(i,j),pvnybsb,0);
         end
         
    end;
 end;




          
        
