%Function for the Asymptotic Variance of the Integrated  CoSpectrum;
% This code computes the Integrated  CoSpectrum as described in
%%----------------------------------------------
%%Pancrazi 2014 ("The Heterogenous Great Moderation", the European Economic
%Review)
%%-----------------------------------------------------------------
% ****** Please refer to that paper in case you are using these codes.******
%%-----------------------------------------------------------------



%Input: 
% Y series stucked in col (T,k)
% N = the number of intervals in which the interval w1-w2 is divided (exptreme included)
% w1,w1= with w1<w2; interval of the frequency of interest 

       
 function[All]=VarIntCoSpec(Y,w1,w2,N)
 [T,k]=size(Y);

Lint=(w2-w1)/N;

bp=[w1:Lint:w2]'; %dimension N+1,1

%STORE THE AUTOCOVARIANCES
RALL=zeros(k,k*T);

for j=0:T-1
    RALL(:,j*k+1:(j+1)*k)=VECTORautocov(Y,j);
end;

%The integral is approximated in the left extreme
SumOutside=zeros(k,k);

for q=1:N
    SumInside=zeros(k,k);

    for j=1-T:T-1        
    if j>-1
        R=RALL(:,j*k+1:(j+1)*k);
    else
        R=(RALL(:,(-j*k+1:(-j+1)*k)))';
    end
    SumInside=SumInside+R.*exp(-1i*bp(q,1)*j);

    end;
    Inside=(real(SumInside./(2*pi))).^2;
    SumOutside=SumOutside+Inside*Lint;
end;

All=real(4*pi*SumOutside);






    
